/*******************************************************************************
* Copyright 2007 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.Text;
using SPSTools;

namespace CreateDefect
{
   class CustomFieldFetch { public string m_strId; public string m_strVal; public CustomFieldFetch(string p_strId, string p_strVal) { m_strId = p_strId; m_strVal = p_strVal; } }

   class JiraConnector
   {
      protected Logger m_logger = null;
      protected JiraSoap.JiraSoapServiceService m_engine = null;
      protected string m_strSession = null;

      public JiraConnector(Logger p_logger, string p_strUrl)
      {
         m_logger = p_logger;

         m_engine = new JiraSoap.JiraSoapServiceService();
         if (p_strUrl.Length > 0) { m_engine.Url = p_strUrl; }
      }

      public bool run(string p_strProject, string p_strUser, string p_strPwd, Hashtable p_hash)
      {
         if (p_hash == null) { m_logger.log("No value data given, so defect creation will fail."); return (false); }
         bool fResult = false;
         try
         {
            m_strSession = m_engine.login(p_strUser, p_strPwd);

            JiraSoap.RemoteProject project = getProject(p_strProject);
            if (project == null) { m_logger.log("Project '" + p_strProject + "' not found, so defect creation will fail."); return (false); }

            JiraSoap.RemoteIssue issueNew = new JiraSoap.RemoteIssue();

            // Set the core fields, which are all required.
            issueNew.project = project.key;
            issueNew.type = getType((string)p_hash["Type"]);
            issueNew.summary = (string)p_hash["Summary"];
            issueNew.assignee = project.lead;

            // Set additional fields, as needed.
            issueNew.description = renderSteps((string)p_hash["Description"]);

            // Add custom field data, if the fields exist in Jira. This could be much less hard-coded, but it'll work for now...
            ArrayList list = new ArrayList();
            JiraSoap.RemoteField fld = getCustomField("Test Run #");
            if (fld != null) { list.Add(new CustomFieldFetch(fld.id, (string)p_hash["Number"])); }
            fld = getCustomField("Test Run Url");
            if (fld != null) { list.Add(new CustomFieldFetch(fld.id, (string)p_hash["Url"])); }

            issueNew.customFieldValues = new JiraSoap.RemoteCustomFieldValue[list.Count];
            for (int i = 0; i < list.Count; ++i)
            {
               issueNew.customFieldValues[i] = new JiraSoap.RemoteCustomFieldValue();
               issueNew.customFieldValues[i].customfieldId = ((CustomFieldFetch)list[i]).m_strId;
               issueNew.customFieldValues[i].values = new string[1];
               issueNew.customFieldValues[i].values[0] = ((CustomFieldFetch)list[i]).m_strVal;
            }

            JiraSoap.RemoteIssue issueCreated = m_engine.createIssue( m_strSession, issueNew);
            m_logger.log("Successfully created issue '" + issueCreated.key + "'");
         }
         catch (Exception p_exception)
         {
            m_logger.log("Creating issue failed: " + p_exception.Message);
         }
         finally { m_engine.logout(m_strSession); }
         return (fResult);
      }

      protected JiraSoap.RemoteProject getProject(string p_strName)
      {
         JiraSoap.RemoteProject proj = null;
         try
         {
            JiraSoap.RemoteProject[] a = m_engine.getProjects( m_strSession);
            for (int i = 0; a != null && i < a.Length; ++i) { if (a[i].name == p_strName) { proj = a[i]; break; } }
         }
         catch (Exception p_exception)
         {
            m_logger.log("Unable to find project '" + p_strName + "': " + p_exception.Message);
         }
         return (proj);
      }

      protected string getType(string p_strName)
      {
         string strKey = "";
         try
         {
            JiraSoap.RemoteIssueType[] a = m_engine.getIssueTypes(m_strSession);
            for (int i = 0; a != null && i < a.Length; ++i) { if (a[i].name == p_strName) { strKey = a[i].id; break; } }
         }
         catch (Exception p_exception)
         {
            m_logger.log("Unable to find issue type '" + p_strName + "': " + p_exception.Message);
         }
         return (strKey);
      }

      protected JiraSoap.RemoteField getCustomField(string p_strFieldName)
      {
         JiraSoap.RemoteField fld = null;
         try
         {
            JiraSoap.RemoteField[] a = m_engine.getCustomFields(m_strSession);
            for (int i = 0; a != null && i < a.Length; ++i) { if (a[i].name == p_strFieldName) { fld = a[i]; break; } }
         }
         catch (Exception p_exception)
         {
            m_logger.log("[WARNING] Unable to find '" + p_strFieldName + "' custom field, that link will not be created");
         }
         return (fld);
      }

      // If we can pull the rendering plugin for the field, we could dynamically translate TTS language into user's Jira language.
      protected string renderSteps(string p_str)
      {  // Assumes generic text rendering, so not much in the way of formatting.
         string str = "";
         try
         {
            str = p_str.Replace("*", "\n*"); // Step
            str = str.Replace("+", "\n   +"); // Expected Result
            str = str.Replace("#", "\n      #"); // Comment
            str = str.TrimStart('\n').Trim();
         }
         catch (Exception p_exception) { str = p_str; }
         return (str);
      }
   }
}
